<?php
/**
 * class BuddyPress
 *
 * Manages BuddyPress related tasks
 *
 * @package DaReactions
 *
 * @since 1.1.0
 */
namespace DaReactions\Plugins;
use BP_Activity_Activity;
use DaReactions\Data;
use DaReactions\Frontend;
use DaReactions\Options;
use Freemius_Exception;
use stdClass;
/**
 * class BuddyPress
 *
 * Manages BuddyPress related tasks
 *
 * @package DaReactions
 *
 * @since 1.1.0
 */
class BuddyPress {
    /**
     * @var string $options_group
     * The name of the group for saved options
     *
     * @since 1.1.0
     */
    private $options;
    /**
     * BuddyPress constructor.
     *
     * @since 1.1.0
     */
    public function __construct() {
        $this->options = Options::getInstance( 'general' );
    }
    /**
     * Add action to BuddyPress Stream
     *
     * @param int $item_id
     * @param string $item_type
     * @param int $reaction_id
     *
     * @throws Freemius_Exception
     * @since 1.3.0
     *
     */
    public function addActionToStream(
            $item_id,
            $item_type,
            $reaction_id
    )
    {
        $reaction       = Data::getReactionById( $reaction_id );
        $reaction_label = $reaction->label;
        $logged_user_id   = bp_loggedin_user_id();
        $logged_user_name = + $logged_user_id === 0 ? __( 'Someone', 'da-reactions' ) : bp_core_get_userlink( $logged_user_id );
        $primary_link = '';
        $action_text = '';
        $target_user_id = 0;
        if ( post_type_exists( $item_type ) ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromRegisteredPostType(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        } else if ( $item_type === 'comment' ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromComment(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        } else if ( $item_type === 'bp_activity' ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromBpActivity(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        } else if ( $item_type === 'bp_activity_comment' ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromBpActivityComment(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        } else if ( $item_type === 'bp_group' ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromBpGroup(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        } else if ( $item_type === 'bp_profile' ) {
            [$primary_link, $action_text] =
                $this->getStreamInfoFromBpProfile(
                    $item_type,
                    $item_id,
                    $logged_user_name,
                    $reaction_label
                );
        }
        $activity_type = 'reaction_added';
        $args = array(
            'action'            => $action_text,
            'component'         => 'reactions',
            'type'              => $activity_type,
            'primary_link'      => $primary_link,
            'user_id'           => $logged_user_id,
            'item_id'           => $item_id,
            'secondary_item_id' => $target_user_id,
            'recorded_time'     => bp_core_current_time(),
            'hide_sitewide'     => false,
            'is_spam'           => false,
            'error_type'        => 'bool'
        );
        bp_activity_add( $args );
    }
    /**
     * Inject reaction buttons HTML in activity content
     *
     * @return false
     *
     * @since 1.1.0
     */
    public function addButtonsToActivity__premium_only() {
        $enabled = $this->options->getOption( 'bp_activity_enabled', 'off' ) === 'on';
        if ( ! $enabled ) {
            return false;
        }
        $item_type = 'bp_activity';
        $item_id   = bp_get_activity_id();
        $reactions_html = Frontend::getButtonsPlaceholder( $item_type, $item_id );
        echo $reactions_html;
        return false;
    }
    /**
     * Inject reaction buttons HTML in activity’s comment content
     *
     * @return void
     *
     * @since 1.1.0
     */
    public function addButtonsToActivityComment__premium_only() {
        global $activities_template;
        $enabled = $this->options->getOption( 'bp_activity_comment_enabled', 'off' ) === 'on';
        if ( ! $enabled ) {
            return;
        }
        $item_type = 'bp_activity_comment';
        $item_id   = $activities_template->activity->current_comment->id;
        $reactions_html = Frontend::getButtonsPlaceholder( $item_type, $item_id );
        echo $reactions_html;
    }
    /**
     * Inject reaction buttons HTML in profile content
     *
     *
     * @since 1.1.0
     */
    public function addButtonsToProfile__premium_only() {
        $enabled = $this->options->getOption( 'bp_profile_enabled', 'off' ) === 'on';
        if ( ! $enabled ) {
            return;
        }
        $item_type = 'bp_profile';
        $item_id   = bp_displayed_user_id();
        $reactions_html = Frontend::getButtonsPlaceholder( $item_type, $item_id );
        echo $reactions_html;
    }
    /**
     * Inject reaction buttons HTML in group content
     *
     *
     * @since 1.1.0
     */
    public function addButtonsToGroup__premium_only() {
        $enabled = $this->options->getOption( 'bp_group_enabled', 'off' ) === 'on';
        if ( ! $enabled ) {
            return;
        }
        $item_type = 'bp_group';
        $item_id   = bp_get_group_id();
        $reactions_html = Frontend::getButtonsPlaceholder( $item_type, $item_id );
        echo $reactions_html;
    }
    /**
     * Modify Where Condition to query reactions among user’s Activities
     *
     * @param $where_conditions
     *
     * @return array
     *
     * @since 1.3.2
     */
    public function addReactionsActivitiesToStream__premium_only( $where_conditions ) {
        if ( isset( $where_conditions['scope_query_sql'] ) ) {
            $pattern = "/user_id[a-z =]+'(\d+)'/";
            if ( preg_match( $pattern, $where_conditions['scope_query_sql'], $match ) ) {
                $user_id = (int) $match[1];
                if ( $user_id > 0 ) {
                    $where_conditions['scope_query_sql'] = "(
                    " . $where_conditions['scope_query_sql'] . "
                        OR (
                            a.component = 'reactions'
                            AND
                            a.secondary_item_id = '" . $user_id . "'
                        )
                    )";
                }
            }
        }
        return $where_conditions;
    }
    /**
     * Render function for Activity Enable checkbox on settings page
     *
     * @since 1.1.0
     */
    public function renderActivityEnableCheckbox() {
        $field_name  = $this->options->getFieldName( 'bp_activity_enabled' );
        $saved_value = $this->options->getOption( 'bp_activity_enabled' );
        $comment_field_name  = $this->options->getFieldName( 'bp_activity_comment_enabled' );
        $comment_saved_value = $this->options->getOption( 'bp_activity_comment_enabled' );
        if ( bp_is_active( 'activity' ) ) {
            $checked         = isset( $saved_value ) && $saved_value === 'on';
            $comment_checked = isset( $comment_saved_value ) && $comment_saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked( $checked, 1 ) ?>
                       value="on"/>
                <label for="id_<?= $field_name ?>"><?php _e( 'Add reactions to BuddyPress’ activities.', 'da-reactions' ); ?></label>
            </p>
            <p>
                <input type="hidden" name="<?= $comment_field_name ?>" value="off"/>
                <input id="id_<?= $comment_field_name ?>" type="checkbox"
                       name="<?= $comment_field_name ?>" <?php checked( $comment_checked, 1 ) ?>
                       value="on"/>
                <label for="id_<?= $comment_field_name ?>"><?php _e( 'Add reactions to BuddyPress’ activity comments too.', 'da-reactions' ); ?></label>
            </p>
            <?php
        } else {
            ?>
            <p class="description">
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>_disabled" type="checkbox" disabled="disabled"/>
                <label for="id_<?= $field_name ?>"><?php _e( 'Enable BuddyPress’ activities before adding reactions to them.', 'da-reactions' ); ?></label>
            </p>
            <?php
        }
    }
    /**
     * Render function for Group Enable checkbox on settings page
     *
     * @since 1.1.0
     */
    public function renderGroupEnableCheckbox() {
        $field_name  = $this->options->getFieldName( 'bp_group_enabled' );
        $saved_value = $this->options->getOption( 'bp_group_enabled' );
        if ( bp_is_active( 'groups' ) ) {
            $checked = isset( $saved_value ) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked( $checked, 1 ) ?>
                       value="on"/>
                <label for="id_<?= $field_name ?>"><?php _e( 'Add reactions to BuddyPress’ groups.', 'da-reactions' ); ?></label>
            </p>
            <?php
        } else {
            ?>
            <p class="description">
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>_disabled" type="checkbox" disabled="disabled"/>
                <label for="id_<?= $field_name ?>_disabled"><?php _e( 'Enable BuddyPress’ groups before adding reactions to them.', 'da-reactions' ); ?></label>
            </p>
            <?php
        }
    }
    /**
     * @param int $activity_id
     *
     * @return array|object|stdClass|void|null
     */
    public static function getActivityById( $activity_id ) {
        global $wpdb;
        $activity_table = $wpdb->prefix . 'bp_activity';
        return $wpdb->get_row(
            "SELECT * FROM $activity_table WHERE `id` = $activity_id AND `component` = 'activity'"
        );
    }
    /**
     * @param int $activity_comment_id
     *
     * @return array|object|stdClass|void|null
     */
    public static function getActivityCommentById( $activity_comment_id ) {
        global $wpdb;
        $activity_table = $wpdb->prefix . 'bp_activity';
        return $wpdb->get_row(
            "SELECT * FROM $activity_table WHERE `id` = $activity_comment_id AND `component` = 'activity' AND type  = 'activity_comment'"
        );
    }
    /**
     * @param int $group_id
     *
     * @return array|object|stdClass|void|null
     */
    public static function getGroupById( $group_id ) {
        global $wpdb;
        $groups_table = $wpdb->prefix . 'bp_groups';
        return $wpdb->get_row(
            "SELECT * FROM $groups_table WHERE `id` = $group_id"
        );
    }
    private function getStreamInfoFromRegisteredPostType($item_type, $item_id, $logged_user_name, $reaction_label): array
    {
        $post_type         = get_post_type_object( $item_type );
        $post              = get_post( $item_id );
        $target_user_id    = $post->post_author;
        $target_user_name  = + $target_user_id === 0 ? __( 'Someone', 'da-reactions' ) : bp_core_get_userlink( $target_user_id );
        $content_permalink = get_permalink( $item_id );
        $action_text = '';
        $primary_link = $content_permalink;
        if ( isset( $post_type ) ) {
            if ( empty( $content_permalink ) ) {
                $content_type_name = $post_type->labels->singular_name;
            } else {
                $content_type_name = '<a href="' . $content_permalink . '" title="' . esc_attr( $post->post_title ) . '">' . $post_type->labels->singular_name . ': ' . $post->post_title . '</a>';
            }
            $action_text = sprintf(
                _x(
                    '%1$s added a %2$s reaction to %3$s',
                    '[current user] added a [reaction name] reaction to [post title]',
                    'da-reactions'
                ),
                $logged_user_name,
                $reaction_label,
                $target_user_name,
                $content_type_name
            );
        }
        return [$primary_link, $action_text];
    }
    private function getStreamInfoFromComment(
            $item_type,
            $item_id,
            $logged_user_name,
            $reaction_label
    ) {
        $comment = get_comment( $item_id );
        $target_user_id   = $comment->user_id;
        $target_user_name = + $target_user_id === 0 ? __( 'Someone', 'da-reactions' ) : bp_core_get_userlink( $target_user_id );
        $content_permalink = get_comment_link( $item_id );
        $content_type_name = __( 'Comment', 'da-reactions' );
        $comment_permalink = get_comment_link( $comment );
        $primary_link      = $comment_permalink;
        $post_link = '<a href="' . get_permalink( $comment->comment_post_ID ) . '">' . get_the_title( $comment->comment_post_ID ) . '</a>';
        if ( ! empty( $content_permalink ) ) {
            $content_type_name = '<a href="' . $content_permalink . '" title="' . __( 'Read comment', 'da-reactions' ) . '">' . $content_type_name . '</a>';
        }
        $action_text = sprintf(
            _x(
                '%1$s added a %2$s reaction to %3$s’s %4$s on %5$s',
                '[current user] added a [reaction name] reaction to [target user]’s [comment] on [post title]',
                'da-reactions'
            ),
            $logged_user_name,
            $reaction_label,
            $target_user_name,
            $content_type_name,
            $post_link
        );
        return [$primary_link, $action_text];
    }
    private function getStreamInfoFromBpActivity(
            $item_type,
            $item_id,
            $logged_user_name,
            $reaction_label
    ) {
        $activity = bp_activity_get( array( 'in' => $item_id ) )['activities'];
        $target_user_name = __( 'Someone', 'da-reactions' );
        $content_type_name = '';
        if ( count( $activity ) === 1 ) {
            $activity       = $activity[0];
            $target_user_id = $activity->user_id;
            $target_user_name = bp_core_get_userlink( $target_user_id );
            /**
             * Cannot use match because Freemius validator does not know the statement
             */
            $content_type_labels = [
                'reaction_added' => __('a Reaction notice', 'da-reactions'),
                'activity_comment' =>__('an Activity comment', 'da-reactions'),
                'activity_update' => __('an Activity update', 'da-reactions'),
                'created_group' => __('a Group creation notice', 'da-reactions'),
                'friendship_created' => __('a Friendship notice', 'da-reactions'),
                'joined_group' => __('a Group notice', 'da-reactions'),
                'new_avatar' => __('an Avatar update', 'da-reactions'),
                'new_blog_comment' => __('a Blog Comment notice', 'da-reactions'),
                'new_blog_post' => __('a Blog Post notice', 'da-reactions'),
                'new_member' => __('a new Member notice', 'da-reactions'),
                'updated_profile' => __('a Profile update', 'da-reactions'),
                'bbp_topic_create' => __('a Forum Topic notice', 'da-reactions'),
                'bbp_reply_create' => __('a Forum Reply notice', 'da-reactions')
            ];
            $content_type_name =
                isset($content_type_labels[$activity->type]) ?
                    $content_type_labels[$activity->type] :
                    __('an Activity', 'da-reactions');
        }
        $content_permalink = bp_activity_get_permalink( $item_id );
        $primary_link = $content_permalink;
        if ( ! empty( $content_permalink ) ) {
            $content_type_name = sprintf("<a href=\"%s\">%s</a>", $content_permalink, $content_type_name);
        }
        $action_text = sprintf(
            _x(
                '%1$s added a %2$s reaction to %3$s on %4$s’s Stream',
                '[current user] added a [reaction name] reaction to [an activity] on [user]’s Stream',
                'da-reactions'
            ),
            $logged_user_name,
            $reaction_label,
            $content_type_name,
            $target_user_name
        );
        return [$primary_link, $action_text];
    }
    private function getStreamInfoFromBpActivityComment(
            $item_type,
            $item_id,
            $logged_user_name,
            $reaction_label
    ) {
        $activity          = new BP_Activity_Activity( $item_id );
        $content_permalink = bp_activity_get_permalink( $item_id );
        $target_user_id    = $activity->user_id;
        $target_user_name  = + $target_user_id === 0 ? __( 'Someone', 'da-reactions' ) : bp_core_get_userlink( $target_user_id );
        $primary_link = $content_permalink;
        if ( empty( $content_permalink ) ) {
            $content_type_name = __( 'Activity comment', 'da-reactions' );
        } else {
            $content_type_name = '<a href="' . $content_permalink . '">' . __( 'Comment', 'da-reactions' ) . '</a>';
        }
        $action_text = sprintf(
            _x(
                '%1$s added a %2$s reaction to %3$s’s %4$s on %5$s',
                '[current user] added a [reaction name] reaction to [target user]’s [comment] on [activity]',
                'da-reactions'
            ),
            $logged_user_name,
            $reaction_label,
            $target_user_name,
            $content_type_name,
            'Activity'
        );
        return [$primary_link, $action_text];
    }
    private function getStreamInfoFromBpGroup(
            $item_type,
            $item_id,
            $logged_user_name,
            $reaction_label
    )
    {
        $group             = groups_get_group( $item_id );
        $content_permalink = bp_get_group_permalink( $group );
        $primary_link = $content_permalink;
        $content_name = $group->name;
        if ( empty( $content_name ) ) {
            $content_name = __( 'Group', 'da-reactions' );
        }
        if ( empty( $content_permalink ) ) {
            $content_type_name = $content_name;
        } else {
            $content_type_name = '<a href="' . $content_permalink . '">' . $content_name . '</a>';
        }
        $action_text = sprintf(
            _x(
                '%1$s added a %2$s reaction to group %3$s',
                '[current user] added a [reaction name] reaction to group [group name]',
                'da-reactions'
            ),
            $logged_user_name,
            $reaction_label,
            $content_type_name
        );
        return [$primary_link, $action_text];
    }
    private function getStreamInfoFromBpProfile(
            $item_type,
            $item_id,
            $logged_user_name,
            $reaction_label
    ) {
        $target_user_id   = $item_id;
        $target_user_link = bp_core_get_userlink( $target_user_id );
        $target_user_name = + $target_user_id === 0 ? __( 'Someone', 'da-reactions' ) : $target_user_link;
        $primary_link = $target_user_link;
        $content_type_name = __( 'Profile', 'da-reactions' );
        $action_text = sprintf(
            _x(
                '%1$s added a %2$s reaction to %3$s’s %4$s',
                '[current user] added a [reaction name] reaction to [target user]’s [profile]',
                'da-reactions'
            ),
            $logged_user_name,
            $reaction_label,
            $target_user_name,
            $content_type_name
        );
        return [$primary_link, $action_text];
    }
}
